; Syntax: NASM, tab width = 8 spaces (or higher)
; - 8bitbubsy, 2023-2025

	CPU 8086
	ORG 100h

VIDEO_SEG	EQU 0A000h
SCREEN_W 	EQU 320
SCREEN_H 	EQU 200
R_COL_OFFSET	EQU (2+(63*0))
G_COL_OFFSET	EQU (2+(63*1))
B_COL_OFFSET	EQU (2+(63*2))
C_COL_OFFSET	EQU (2+(63*3))
KEY_ESC		EQU 27

	push cs
	pop ds
	; -------------------
	call detectVGAWithColorMonitor
	jnc .L0
	lea dx,errorText	; print error msg
	mov ah,9
	int 21h
	jmp exitToDOS
.L0:	; -------------------	; get current video mode
	mov ax,0F00h
	int 10h
	mov [oldVideoMode],al
	; -------------------	; enter 320x200 @ 70Hz (chunky mode)
	mov ax,0013h
	int 10h
	; -------------------	; set black color palette
	mov ax,1
	xor bx,bx
	xor cx,cx
	xor dx,dx
	call setPaletteColor
	; -------------------	; set other palette entries
	call setColorGradientsPalette
	call setGrayscalePalette
	; -------------------
	mov ax,VIDEO_SEG
	mov es,ax
	; -------------------
	call drawGradientBars
	; -------------------
.loop:	mov ah,8		; get key
	int 21h
	cmp al,KEY_ESC
	je .done
	call switchTestMode
	jmp .loop
.done:	; -------------------	; set back old video mode
	xor ax,ax
	mov al,[oldVideoMode]
	int 10h
exitToDOS:
	mov ah,4Ch
	int 21h

detectVGAWithColorMonitor:
	mov ax,1A00h
	int 10h
	cmp al,26
	jne .error
	cmp bl,8
	je .ok
	cmp bl,10
	je .ok
	cmp bl,12
	je .ok
.error:	stc
	ret
.ok:	clc
	ret

waitForVblank:
	mov dx,3DAh
.L0:	in al,dx
	test al,8
	jz .L0
.L1:	in al,dx
	test al,8
	jnz .L1
	ret

	; AL = index, BL = red, CL = green, DL = blue
setPaletteColor:
	push ax
	push dx
	mov bp,sp
	mov dx,3C8h
	mov ax,[bp+2]
	out dx,al
	inc dx
	mov al,bl
	out dx,al
	mov al,cl
	out dx,al
	mov ax,[bp+0]
	out dx,al
	pop dx
	pop ax
	ret

%macro SET_PAL 4
	mov al,%1
	mov bl,%2
	mov cl,%3
	mov dl,%4
	call setPaletteColor
%endmacro

switchTestMode:
	mov al,[testMode]
	inc al
	cmp al,6
	jne .L0
	xor al,al
.L0:	mov [testMode],al
	; -------------------
	cmp al,0
	jne .L1
	call drawGradientBars
	ret
.L1:	cmp al,1
	jne .L2
	call drawColorBars
	ret
.L2:	cmp al,2
	jne .L3
	; -------------------	; red screen
	xor di,di		; clear screen first
	xor ax,ax
	mov cx,(SCREEN_W*SCREEN_H)/2
	cld
	rep stosw
	; -------------------
	call waitForVblank
	SET_PAL 0,63,0,0
	ret
.L3:	cmp al,3
	jne .L4
	; -------------------	; green screen
	call waitForVblank
	SET_PAL 0,0,63,0
	ret
.L4:	cmp al,4
	jne .L5
	; -------------------	; blue screen
	call waitForVblank
	SET_PAL 0,0,0,63
.L5:	cmp al,5
	jne .L6
	; -------------------	; white screen
	call waitForVblank
	SET_PAL 0,63,63,63
.L6:	ret

setGrayscalePalette:
	mov si,62
.loop:	lea ax,[si+C_COL_OFFSET]
	mov bx,si
	mov cx,si
	mov dx,si
	call setPaletteColor
	dec si
	jns .loop
	ret

setColorGradientsPalette:
	mov si,62
.loop:	; -------------------	; red gradient
	lea ax,[si+R_COL_OFFSET]
	mov bx,si
	xor cx,cx
	xor dx,dx
	call setPaletteColor
	; -------------------	; green gradient
	lea ax,[si+G_COL_OFFSET]
	xor bx,bx
	mov cx,si
	xor dx,dx
	call setPaletteColor
	; -------------------	; blue gradient
	lea ax,[si+B_COL_OFFSET]
	xor bx,bx
	xor cx,cx
	mov dx,si
	call setPaletteColor
	; -------------------
	dec si
	jns .loop
	ret

drawGradientBar:
	mov si,(SCREEN_W/5)-1	; 5 (320/64) pixels per color
	cld
.yloop:	mov cx,si
	mov ax,0101h ; use special palette entry for black entry (kludge)
	stosw
	stosw
	stosb
	mov ax,bx
.xloop:	mov ah,al
	stosw
	stosw
	stosb	
	inc al
	dec cx
	jne .xloop
	dec dx
	jne .yloop
	ret

drawGradientBars:
	call waitForVblank
	SET_PAL 0,63,63,63	; white background
	call setColorGradientsPalette
	xor di,di
	; -------------------	; red gradient bar
	mov bx,R_COL_OFFSET
	mov dx,50		; height
	call drawGradientBar
	; ------------------	;  green gradient bar
	mov bx,G_COL_OFFSET
	mov dx,50
	call drawGradientBar
	; -------------------	; blue gradient bar
	mov bx,B_COL_OFFSET
	mov dx,50
	call drawGradientBar
	; -------------------	; grayscale gradient bar
	mov bx,C_COL_OFFSET
	mov dx,50
	call drawGradientBar
	ret

drawColorBars:
	call waitForVblank
	SET_PAL 0,0,0,0				; black (background)
	SET_PAL	R_COL_OFFSET+0, 63,  63,  63	; white
	SET_PAL R_COL_OFFSET+1, 63,  63,   0	; yellow
	SET_PAL R_COL_OFFSET+2,  0,  63,  63	; cyan
	SET_PAL R_COL_OFFSET+3,  0,  63,   0	; green
	SET_PAL R_COL_OFFSET+4, 63,   0,  63	; magenta
	SET_PAL R_COL_OFFSET+5, 63,   0,   0	; red
	SET_PAL R_COL_OFFSET+6,  0,   0,  63	; blue
	SET_PAL R_COL_OFFSET+7,  4,   4,   4	; dark gray	
	; -------------------
	xor di,di
	mov si,(SCREEN_W/8)/2
	mov dx,SCREEN_H-50
	cld
.yloop:	mov ax,((R_COL_OFFSET+0)*256)+(R_COL_OFFSET+0)
	mov bx,((R_COL_OFFSET+7)*256)+(R_COL_OFFSET+7)
.xloop:	mov cx,si
	rep stosw
	add ax,0101h
	cmp ax,bx
	jbe .xloop
	dec dx
	jne .yloop
	ret

; -----------------------------------------------------------------------------
; -----------------------------------------------------------------------------

oldVideoMode:	DB 0
testMode:	DB 0

errorText:
	DB "ERROR: This program requires a VGA graphics adapter and a color "
	DB "monitor.",13,10,"$"
